@@ -26,16 +26,19 @@ def load_data(config):
     if config.data_source == 'csv':
         if config.csv_test_data_relative_path:
             try:
-                combined_path = config.this_path / 'predictit' / 'test_data' / config.csv_test_data_relative_path
+                combined_path = config.this_path / 'predictit' / \
+                    'test_data' / config.csv_test_data_relative_path
                 config.csv_full_path = combined_path.as_posix()
             except Exception:
                 raise FileNotFoundError(colorize(
                     "\n\nERROR - Test data load failed - You are using relative path. This continue from folder test_data"
                     "in predictit. If you want to use other folder, keep relative_path empty and use absolute csv_full_path.\n\n"))
         try:
-            data = pd.read_csv(config.csv_full_path, sep=config.csv_style['separator'], decimal=config.csv_style['decimal']).iloc[-config.max_imported_length:, :]
+            data = pd.read_csv(config.csv_full_path, sep=config.csv_style['separator'],
+                               decimal=config.csv_style['decimal']).iloc[-config.max_imported_length:, :]
         except Exception:
-            raise FileNotFoundError(colorize(f"\n ERROR - Test data load failed - Setup CSV adress and column name in config \n\n"))
+            raise FileNotFoundError(colorize(
+                "\n ERROR - Test data load failed - Setup CSV adress and column name in config \n\n"))
 
     ############# Load SQL data #############
     elif config.data_source == 'sql':
@@ -90,7 +93,8 @@ def data_consolidation(data, config):
             predicted_column_name = config.predicted_column
 
             if predicted_column_name not in data_for_predictions_df.columns:
-                raise KeyError(colorize(f"Predicted column name - '{config.predicted_column}' not found in data. Change in config - 'predicted_column'. Available columns: {list(data_for_predictions_df.columns)}"))
+                raise KeyError(colorize(
+                    f"Predicted column name - '{config.predicted_column}' not found in data. Change in config - 'predicted_column'. Available columns: {list(data_for_predictions_df.columns)}"))
 
         else:
             predicted_column_name = data_for_predictions_df.columns[config.predicted_column]
@@ -98,7 +102,8 @@ def data_consolidation(data, config):
         try:
             int(data_for_predictions_df[predicted_column_name].iloc[0])
         except Exception:
-            raise KeyError(colorize(f"Predicted column is not number datatype. Setup correct 'predicted_column' in config.py. Available columns: {list(data_for_predictions_df.columns)}"))
+            raise KeyError(colorize(
+                f"Predicted column is not number datatype. Setup correct 'predicted_column' in config.py. Available columns: {list(data_for_predictions_df.columns)}"))
 
         if config.datetime_index not in [None, '']:
 
@@ -107,34 +112,44 @@ def data_consolidation(data, config):
                     pass
                 else:
                     if isinstance(config.datetime_index, str):
-                        data_for_predictions_df.set_index(config.datetime_index, drop=True, inplace=True)
+                        data_for_predictions_df.set_index(
+                            config.datetime_index, drop=True, inplace=True)
 
                     else:
-                        data_for_predictions_df.set_index(data_for_predictions_df.columns[config.datetime_index], drop=True, inplace=True)
+                        data_for_predictions_df.set_index(
+                            data_for_predictions_df.columns[config.datetime_index], drop=True, inplace=True)
 
-                    data_for_predictions_df.index = pd.to_datetime(data_for_predictions_df.index)
+                    data_for_predictions_df.index = pd.to_datetime(
+                        data_for_predictions_df.index)
 
             except Exception:
-                raise KeyError(colorize(f"Datetime name / index from config - '{config.datetime_index}' not found in data or not datetime format. Change in config - 'datetime_index'. Available columns: {list(data_for_predictions_df.columns)}"))
+                raise KeyError(colorize(
+                    f"Datetime name / index from config - '{config.datetime_index}' not found in data or not datetime format. Change in config - 'datetime_index'. Available columns: {list(data_for_predictions_df.columns)}"))
 
             if config.freq:
                 data_for_predictions_df.sort_index(inplace=True)
                 if config.resample_function == 'sum':
-                    data_for_predictions_df = data_for_predictions_df.resample(config.freq).sum()
+                    data_for_predictions_df = data_for_predictions_df.resample(
+                        config.freq).sum()
                 if config.resample_function == 'mean':
-                    data_for_predictions_df = data_for_predictions_df.resample(config.freq).mean()
-                data_for_predictions_df = data_for_predictions_df.asfreq(config.freq, fill_value=0)
+                    data_for_predictions_df = data_for_predictions_df.resample(
+                        config.freq).mean()
+                data_for_predictions_df = data_for_predictions_df.asfreq(
+                    config.freq, fill_value=0)
 
             else:
 
-                data_for_predictions_df.index.freq = pd.infer_freq(data_for_predictions_df.index)
+                data_for_predictions_df.index.freq = pd.infer_freq(
+                    data_for_predictions_df.index)
 
                 if data_for_predictions_df.index.freq is None:
-                    user_warning("Datetime index was provided from config, but frequency guess failed. Specify 'freq' in config to resample and have equal sampling if you want.")
+                    user_warning(
+                        "Datetime index was provided from config, but frequency guess failed. Specify 'freq' in config to resample and have equal sampling if you want.")
                     data_for_predictions_df.reset_index(inplace=True)
 
         # Make predicted column index 0
-        data_for_predictions_df.insert(0, predicted_column_name, data_for_predictions_df.pop(predicted_column_name))
+        data_for_predictions_df.insert(
+            0, predicted_column_name, data_for_predictions_df.pop(predicted_column_name))
 
         data_for_predictions_df = data_for_predictions_df.iloc[-config.datalength:, :]
 
@@ -146,22 +161,25 @@ def data_consolidation(data, config):
                 if config.remove_nans == 'any_rows':
                     axis = 0
 
-                data_for_predictions_df.dropna(how='any', inplace=True, axis=axis)
+                data_for_predictions_df.dropna(
+                    how='any', inplace=True, axis=axis)
 
-            data_for_predictions_df = data_for_predictions_df.select_dtypes(include='number')
+            data_for_predictions_df = data_for_predictions_df.select_dtypes(
+                include='number')
             data_for_predictions = data_for_predictions_df.values
 
         else:
-            data_for_predictions = data_for_predictions_df[predicted_column_name].values.reshape(-1, 1)
+            data_for_predictions = data_for_predictions_df[predicted_column_name].values.reshape(
+                -1, 1)
 
     ### np.ndarray ###
 
-
     elif isinstance(data, np.ndarray):
         data_for_predictions = data.copy()
 
         if not isinstance(config.predicted_column, int):
-            raise TypeError(colorize("'predicted_column' in config is a string and data in numpy array format. Numpy does not allow string assignment"))
+            raise TypeError(colorize(
+                "'predicted_column' in config is a string and data in numpy array format. Numpy does not allow string assignment"))
 
         predicted_column_name = 'Predicted column'
 
@@ -175,22 +193,26 @@ def data_consolidation(data, config):
 
         # Make predicted column on index 0
         if config.other_columns and config.predicted_column != 0 and data_for_predictions.shape[1] != 1:
-            data_for_predictions[:, [0, config.predicted_column]] = data_for_predictions[:, [config.predicted_column, 0]]
+            data_for_predictions[:, [0, config.predicted_column]
+                                 ] = data_for_predictions[:, [config.predicted_column, 0]]
 
         if not config.other_columns:
             data_for_predictions = data_for_predictions[:, 0].reshape(-1, 1)
 
         data_for_predictions_df = pd.DataFrame(data_for_predictions)
-        data_for_predictions_df.rename(columns={data_for_predictions_df.columns[0]: predicted_column_name}, inplace=True)
+        data_for_predictions_df.rename(
+            columns={data_for_predictions_df.columns[0]: predicted_column_name}, inplace=True)
 
     else:
         raise TypeError(colorize("""Input data must be in pd.dataframe, pd.series or numpy array. If you use csv or sql data source, its converted automatically,
                                  but setup csv_full_path. Check config comments fore  more informations..."""))
 
     try:
-        data_for_predictions = data_for_predictions.astype(config.dtype, copy=False)
+        data_for_predictions = data_for_predictions.astype(
+            config.dtype, copy=False)
     except Exception:
-        user_warning(f"Predicted column - {config.predicted_column} is not a number. Check config")
+        user_warning(
+            f"Predicted column - {config.predicted_column} is not a number. Check config")
 
     return data_for_predictions, data_for_predictions_df, predicted_column_name
 
@@ -199,27 +221,35 @@ def keep_corelated_data(data, predicted_column_index=0, threshold=0.5):
     """Remove columns that are not corelated enough to predicted columns.
 
     Args:
-        data (np.array): Time series data.
+        data (np.array, pd.DataFrame): Time series data.
         predicted_column_index (int, optional): Column that is predicted. Defaults to 0.
         threshold (float, optional): After correlation matrix is evaluated, all columns that are correlated less than threshold are deleted. Defaults to 0.2.
 
     Returns:
-        np.array: Data with no columns that are not corelated with predicted column.
+        np.array, pd.DataFrame: Data with no columns that are not corelated with predicted column.
     """
 
-    # If some row have no variance - RuntimeWarning warning in correlation matrix computing and then in comparing
-    with warnings.catch_warnings():
-        warnings.simplefilter("ignore", RuntimeWarning)
+    if isinstance(data, np.ndarray):
+        # If some row have no variance - RuntimeWarning warning in correlation matrix computing and then in comparing
+        with warnings.catch_warnings():
+            warnings.simplefilter("ignore", RuntimeWarning)
 
-        corr = np.corrcoef(data.T)
-        corr = np.nan_to_num(corr, 0)
+            corr = np.corrcoef(data.T)
+            corr = np.nan_to_num(corr, 0)
 
-        range_array = np.array(range(corr.shape[0]))
-        columns_to_del = range_array[abs(corr[predicted_column_index]) <= threshold]
+            range_array = np.array(range(corr.shape[0]))
+            columns_to_del = range_array[abs(
+                corr[predicted_column_index]) <= threshold]
 
-        data = np.delete(data, columns_to_del, axis=0)
+            data = np.delete(data, columns_to_del, axis=0)
 
-        return data
+    elif isinstance(data, pd.DataFrame):
+        corr = data.corr()
+        names_to_del = list(
+            corr[corr[corr.columns[predicted_column_index]] <= abs(0.6)].index)
+        data.drop(columns=names_to_del, inplace=True)
+
+    return data
 
     # if isinstance(data, pd.DataFrame):
     #     corr = data.corr()
@@ -228,10 +258,10 @@ def keep_corelated_data(data, predicted_column_index=0, threshold=0.5):
 
 
 def remove_the_outliers(data, predicted_column_index=0, threshold=3):
-    """Remove values far from mean - probably errors.
+    """Remove values far from mean - probably errors. If more columns, then onlyu rows that have outlier on predicted column will be deleted.
 
     Args:
-        data (np.array): Time series data.
+        data (np.array): Time series data. Must have ndim = 2, if univariate, reshape...
         predicted_column_index (int, optional): Column that is predicted. Defaults to 0.
         threshold (int, optional): How many times must be standard deviation from mean to be ignored. Defaults to 3.
 
@@ -245,24 +275,20 @@ def remove_the_outliers(data, predicted_column_index=0, threshold=3):
         [1, 3, 5, 2, 3, 4, 5, 3]
     """
 
-    # if isinstance(data, pd.DataFrame):
+    if isinstance(data, np.ndarray):
+        data_mean = data[:, predicted_column_index].mean()
+        data_std = data[:, predicted_column_index].std()
 
-    #     data_mean = data.iloc[:, predicted_column_index].mean()
-    #     data_std = data.iloc[:, predicted_column_index].std()
+        range_array = np.array(range(data.shape[0]))
+        names_to_del = range_array[abs(
+            data[:, predicted_column_index] - data_mean) > threshold * data_std]
+        data = np.delete(data, names_to_del, axis=0)
 
-    #     data = data[abs(data[data.columns[0]] - data_mean) < threshold * data_std]
+    elif isinstance(data, pd.DataFrame):
+        data_mean = data.iloc[:, predicted_column_index].mean()
+        data_std = data.iloc[:, predicted_column_index].std()
 
-
-    # if data.ndim == 1:
-    #     data = data[abs(data - data.mean()) < threshold * data.std()]
-    #     return data
-
-    data_mean = data[:, predicted_column_index].mean()
-    data_std = data[:, predicted_column_index].std()
-
-    range_array = np.array(range(data.shape[0]))
-    names_to_del = range_array[abs(data[:, predicted_column_index] - data_mean) > threshold * data_std]
-    data = np.delete(data, names_to_del, axis=0)
+        data = data[abs(data[data.columns[0]] - data_mean) < threshold * data_std]
 
     return data
 
@@ -283,7 +309,10 @@ def do_difference(data):
         [2, 2, -3]
     """
 
-    return np.diff(data.T).T
+    if isinstance(data, np.ndarray):
+        return np.diff(data, axis=0)
+    else:
+        return data.diff().iloc[1:]
 
 
 def inverse_difference(differenced_predictions, last_undiff_value):
@@ -377,7 +406,8 @@ def smooth(data, window=101, polynom_order=2):
         ndarray: Cleaned data with less noise.
     """
     for i in range(data.shape[1]):
-        data[:, i] = scipy.signal.savgol_filter(data[:, i], window, polynom_order)
+        data[:, i] = scipy.signal.savgol_filter(
+            data[:, i], window, polynom_order)
 
     return data
 
@@ -433,20 +463,24 @@ def preprocess_data(data_for_predictions, predicted_column_index=0, multicolumn=
             data_for_predictions, predicted_column_index=predicted_column_index, threshold=remove_outliers)
 
     if smoothit:
-        data_for_predictions = smooth(data_for_predictions, smoothit[0], smoothit[1])
+        data_for_predictions = smooth(
+            data_for_predictions, smoothit[0], smoothit[1])
 
     if correlation_threshold and multicolumn:
-        data_for_predictions = keep_corelated_data(data_for_predictions, threshold=correlation_threshold)
+        data_for_predictions = keep_corelated_data(
+            data_for_predictions, threshold=correlation_threshold)
 
     if data_transform == 'difference':
         last_undiff_value = data_for_predictions[-1, 0]
         for i in range(data_for_predictions.shape[1]):
-            data_for_predictions[1:, i] = do_difference(data_for_predictions[:, i])
+            data_for_predictions[1:, i] = do_difference(
+                data_for_predictions[:, i])
     else:
         last_undiff_value = None
 
     if standardizeit:
-        data_for_predictions, final_scaler = standardize(data_for_predictions, used_scaler=standardizeit)
+        data_for_predictions, final_scaler = standardize(
+            data_for_predictions, used_scaler=standardizeit)
     else:
         final_scaler = None
 
